      PROGRAM DRVTRI
C
C     ******************************************************************
C
C                                                           From Padua2D
C
C                                       Marco Caliari and Marco Vianello
C                             Department of Pure and Applied Mathematics
C                                            University of Padua (Italy)
C                                        {mcaliari,marcov}@math.unipd.it
C
C                                                      Stefano De Marchi
C                                         Department of Computer Science
C                                           University of Verona (Italy)
C                                              stefano.demarchi@univr.it
C
C                                                        June 19th, 2007
C
C       This driver computes the interpolation of the Franke function
C     on the triangle T(U,V,W) = T(0,0,1,0,0,1) = T at the first family
C     of Padua points. The degree of interpolation is 50 (see DEG) and 
C     the number of target points is NTG = NTG1 ** 2 = 100 ** 2.
C     The maps from the reference square to the current domain are 
C     SIGMA1 and SIGMA2 with inverses ISIGM1 and ISIGM2.
C
C     ******************************************************************
C
C     LOCALS type                
C     
C     DEGMAX integer                   maximum degree allowed
C     NPDMAX integer                   maximum number of Padua points
C                                      = (DEGMAX + 1) * (DEGMAX + 2) / 2
C     NTGMAX integer                   maximum number of target points
C     DEG    integer                   actual degree
C     NPD    integer                   actual number of Padua points
C                                      = (DEG + 1) * (DEG + 2) / 2
C     NTG    integer                   actual number of target points
C     PD1    double(NPDMAX)            array of the first coordinates 
C                                      of Padua points
C     PD2    double(NPDMAX)            array of the second coordinates
C                                      of Padua points
C     WPD    double(NPDMAX)            array of the weights
C     FPD    double(NPDMAX)            values of the function at 
C                                      Padua points
C     T1     double(0:DEGMAX,DEGMAX)   Normalized Chebyshev polynomials 
C                                      at the first coordinate of the 
C                                      points
C     T2     double(0:DEGMAX,DEGMAX)   Normalized Chebyshev polynomials  
C                                      at the second coordinate of the 
C                                      points
C     C0     double(0:DEGMAX,0:DEGMAX) matrix C0 of the coefficients
C     TG1    double(NTGMAX)            array of the first coordinates 
C                                      of target points
C     TG2    double(NTGMAX)            array of the second coordinates
C                                      of target points
C     INTFTG double(NTGMAX)            values of the interpolated 
C                                      function
C     F      double function           Franke function at target points
C
C     SIGMA1 double function           first component of the map
C                                      [-1,1]^2 -> T
C     SIGMA2 double function           second component of the map
C                                      [-1,1]^2 -> T
C     MAXERR double                    maximum norm of the error at
C                                      target points
C     ESTERR double                    estimated error
C     NTG1   integer                   number of target points for 
C                                      direction
C     
C     ******************************************************************
C
      INTEGER DEGMAX,NPDMAX,NTGMAX
      PARAMETER (DEGMAX = 400,NPDMAX = (DEGMAX + 1) * (DEGMAX + 2) / 2,
     1     NTGMAX = 1000000)
      INTEGER DEG,NPD,NTG
      DOUBLE PRECISION PD1(NPDMAX),PD2(NPDMAX),WPD(NPDMAX),FPD(NPDMAX)
      DOUBLE PRECISION T1(0:DEGMAX,NPDMAX),T2(0:DEGMAX,NPDMAX)
      DOUBLE PRECISION C0(0:DEGMAX,0:DEGMAX)
      DOUBLE PRECISION TG1(NTGMAX),TG2(NTGMAX),INTFTG(NTGMAX)
      DOUBLE PRECISION F,MAXERR,SIGMA1,SIGMA2,PD2VAL,ESTERR,
     1     MINVAL,MAXVAL,MAXDEV,MEAN,ISIGM1,ISIGM2
      DOUBLE PRECISION U1,V1,W1,U2,V2,W2
      DATA U1/0.0D0/,U2/0.0D0/,V1/1.0D0/,V2/0.0D0/,W1/0.D0/,W2/1.0D0/
      EXTERNAL F,SIGMA1,SIGMA2,ISIGM1,ISIGM2,PD2VAL
      INTEGER I,NTG1
      DEG = 50
      NTG1 = 100
      NTG = NTG1 ** 2 
      IF (DEG .GT. DEGMAX) THEN
         WRITE(6,*) 'IN _DRVTRI: DEG ',DEG,' > DEGMAX ',DEGMAX
         WRITE(6,*) 'STOP'
         STOP
      END IF
      IF (NTG1 .LT. 2) THEN
         WRITE(6,*) 'IN _DRVTRI: NTG1',NTG1,' < 2'
         WRITE(6,*) 'STOP'
         STOP
      END IF      
      IF (NTG .GT. NTGMAX) THEN
         WRITE(6,*) 'IN _DRVTRI: NTG ',NTG,' > NTGMAX ',NTGMAX
         WRITE(6,*) 'STOP'
         STOP
      END IF      
C     
C     Build the first family of Padua points
C     
      CALL PDPTS(DEG,PD1,PD2,WPD,NPD)
      OPEN(10,FILE='fpd.dat',STATUS='UNKNOWN')
      WRITE(10,*) '# Function values at ',NPD,' Padua points'
      WRITE(6,*)
      WRITE(6,*) 'Function values at Padua points'//
     1     '               on fpd.dat'
      DO 10 I = 1,NPD
C     
C     Compute the Franke function at Padua points mapped to T
C     
         FPD(I) = F(1,SIGMA1(PD1(I),PD2(I),U1,U2,V1,V2,W1,W2),
     1        SIGMA2(PD1(I),PD2(I),U1,U2,V1,V2,W1,W2))
         WRITE(10,2) SIGMA1(PD1(I),PD2(I),U1,U2,V1,V2,W1,W2),
     1        SIGMA2(PD1(I),PD2(I),U1,U2,V1,V2,W1,W2),FPD(I)
 10    CONTINUE
      WRITE(10,*) '# End'
      CLOSE(10)
C     
C     Compute the matrix C0 of the coefficients in the basis of the 
C     normalized Chebyshev polynomials
C     
      CALL PADUA2(DEG,DEGMAX,NPD,PD1,PD2,WPD,FPD,T1,T2,C0,ESTERR)
C     
C     Build the set of target points on T(U,V,W)
C     
      CALL TARGET(U1,U2,V1,V2,W1,W2,NTG1,TG1,TG2,NTG)
      DO 20 I = 1,NTG
C     
C     Evaluate the interpolation at target points
C     
         INTFTG(I) = PD2VAL(DEG,DEGMAX,C0,
     1        ISIGM1(TG1(I),TG2(I),U1,U2,V1,V2,W1,W2),
     2        ISIGM2(TG1(I),TG2(I),U1,U2,V1,V2,W1,W2),T1,T2)
 20   CONTINUE
C     
C     Compute the error relative to the max deviation from the mean
C     
      MAXERR = 0.0D0
      MEAN = 0.0D0
      MAXVAL = F(1,TG1(1),TG2(1))
      MINVAL = MAXVAL
      OPEN(10,FILE='ftg.dat',STATUS='UNKNOWN')
      WRITE(10,*) '# Function values at ',NTG,' target points'
      WRITE(6,*) 'Function values at target points'//
     1     '              on ftg.dat'
      OPEN(11,FILE='intftg.dat',STATUS='UNKNOWN')
      WRITE(11,*) '# Interpolated function values at ',NTG,
     1     ' target points'
      WRITE(6,*) 'Interpolated function values at target points'//
     1     ' on intftg.dat'
      DO 30 I = 1,NTG
         MAXERR = MAX(MAXERR,ABS(F(1,TG1(I),TG2(I)) - INTFTG(I)))
         MEAN = MEAN + F(1,TG1(I),TG2(I))
         MAXVAL = MAX(MAXVAL,F(1,TG1(I),TG2(I)))
         MINVAL = MIN(MINVAL,F(1,TG1(I),TG2(I)))
         WRITE(10,2) TG1(I),TG2(I),F(1,TG1(I),TG2(I))
         WRITE(11,2) TG1(I),TG2(I),INTFTG(I)
         IF (MOD(I,NTG1).EQ.0) THEN
            WRITE(10,*)
            WRITE(11,*)
         END IF   
 30   CONTINUE   
      IF (MAXVAL .EQ. MINVAL) THEN
         MAXDEV = 1.0D0
      ELSE
         MEAN = MEAN / NTG
         MAXDEV = MAX(MAXVAL - MEAN,MEAN - MINVAL)
      END IF   
      WRITE(10,*) '# End'
      CLOSE(10)
      WRITE(11,*) '# End'
      CLOSE(11)
      WRITE(6,*)
      WRITE(6,1) 'Estimated error:     ',ESTERR / MAXDEV
      WRITE(6,1) 'Current error:       ',MAXERR / MAXDEV
      WRITE(6,1) 'It should be:        ',0.2131552D-07
      WRITE(6,*)
 1    FORMAT(1X,A21,E13.7)
 2    FORMAT(3(1X,E14.8))
      STOP
      END
      DOUBLE PRECISION FUNCTION SIGMA1(T1,T2,U1,U2,V1,V2,W1,W2)
C     
C     This functions returns the first component of the map
C     [-1,1]^2 -> T(U,V,W).
C
      DOUBLE PRECISION T1,T2
      DOUBLE PRECISION U1,U2,V1,V2,W1,W2
      DOUBLE PRECISION PHI,PSI
      EXTERNAL PHI,PSI
      SIGMA1 = (V1 - U1) * PHI(T1,T2) + (W1 - U1) * PSI(T1,T2) + U1
      RETURN
      END
      DOUBLE PRECISION FUNCTION ISIGM1(SIGMA1,SIGMA2,U1,U2,V1,V2,W1,W2)
C     
C     This functions returns the first component of the map
C     T(U,V,W) -> [-1,1]^2.
C
      DOUBLE PRECISION SIGMA1,SIGMA2,U1,U2,V1,V2,W1,W2
      DOUBLE PRECISION RHO1,RHO2
      RHO1 = (SIGMA1 * (W2 - U2) - 
     1     SIGMA2 * (W1 - U1) + (W1 - U1) * U2 - (W2 - U2) * U1) /
     2     ((V1 - U1) * (W2 - U2) - (V2 - U2) * (W1 - U1))
      RHO2 = (SIGMA1 * (V2 - U2) - 
     1     SIGMA2 * (V1 - U1) + (V1 - U1) * U2 - (V2 - U2) * U1) /
     2     ((W1 - U1) * (V2 - U2) - (W2 - U2) * (V1 - U1))
      IF (RHO2 .EQ. 1.0D0) THEN
         ISIGM1 = 0.0D0
      ELSE
         ISIGM1 = 2 * RHO1 / (1 - RHO2) - 1
      END IF   
      RETURN
      END
      DOUBLE PRECISION FUNCTION SIGMA2(T1,T2,U1,U2,V1,V2,W1,W2)
C     
C     This functions returns the second component of the map
C     [-1,1]^2 -> T(U,V,W).
C
      DOUBLE PRECISION T1,T2
      DOUBLE PRECISION U1,U2,V1,V2,W1,W2
      DOUBLE PRECISION PHI,PSI
      EXTERNAL PHI,PSI
      SIGMA2 = (V2 - U2) * PHI(T1,T2) + (W2 - U2) * PSI(T1,T2) + U2
      RETURN
      END
      DOUBLE PRECISION FUNCTION ISIGM2(SIGMA1,SIGMA2,U1,U2,V1,V2,W1,W2)
C     
C     This functions returns the second component of the map
C     T(U,V,W) -> [-1,1]^2.
C
      DOUBLE PRECISION SIGMA1,SIGMA2,U1,U2,V1,V2,W1,W2
      DOUBLE PRECISION RHO2
      RHO2 = (SIGMA1 * (V2 - U2) - 
     1     SIGMA2 * (V1 - U1) + (V1 - U1) * U2 - (V2 - U2) * U1) /
     2     ((W1 - U1) * (V2 - U2) - (W2 - U2) * (V1 - U1))
      IF (RHO2 .EQ. 1.0D0) THEN
         ISIGM2 = 1.0D0
      ELSE
         ISIGM2 = 2 * RHO2 - 1
      END IF   
      RETURN
      END
      DOUBLE PRECISION FUNCTION PHI(X1,X2)
C     
C     This function returns the first component of the 
C     map [-1,1]^2 -> T.
C
      DOUBLE PRECISION X1,X2
      DOUBLE PRECISION V1,V2,B,PSI
      DATA V1/0.0D0/,V2/1.0D0/,B/1.0D0/
      EXTERNAL PSI
      PHI = B / 2 * (X1 + 1) * (1 - PSI(X1,X2) / V2) +
     1     V1 * PSI(X1,X2) / V2
      RETURN
      END
      DOUBLE PRECISION FUNCTION PSI(X1,X2)
C     
C     This function returns the second component of the 
C     map [-1,1]^2 -> T.
C
      DOUBLE PRECISION X1,X2
      DOUBLE PRECISION V2
      DATA V2/1.0D0/
      PSI = V2 * (X2 + 1) / 2
      RETURN
      END
      SUBROUTINE TARGET(U1,U2,V1,V2,W1,W2,NTG1,TG1,TG2,NTG)
C
C     Target points on T(U,V,W).
C     The number of target points is NTG1 * NTG1.
C
      INTEGER NTG1,NTG
      DOUBLE PRECISION U1,U2,V1,V2,W1,W2,TG1(NTG1 ** 2),TG2(NTG1 ** 2)
      INTEGER I,J
      DO 10 I = 1,NTG1
         DO 20 J = 1,NTG1
            TG1((I - 1) * NTG1 + J) = 
     1           (V1 - U1) * (I - 1.D0) / (NTG1 - 1) + 
     2           (W1 - U1) * ((J - 1.D0) / (NTG1 - 1)) * 
     3           (1 - (I - 1.D0) / (NTG1 - 1)) + U1
            TG2((I - 1) * NTG1 + J) = 
     1           (V2 - U2) * (I - 1.D0) / (NTG1 - 1) + 
     2           (W2 - U2) * ((J - 1.D0) / (NTG1 - 1)) * 
     3           (1 - (I - 1.D0) / (NTG1 - 1)) + U2
 20      CONTINUE
 10   CONTINUE
      NTG = NTG1 ** 2
      RETURN
      END
