      SUBROUTINE PADUA2(DEG,DEGMAX,NPD,PD1,PD2,WPD,FPD,T1,T2,C0,ESTERR)
C
C     ******************************************************************
C
C                                                           From Padua2D
C
C                                       Marco Caliari and Marco Vianello
C                             Department of Pure and Applied Mathematics
C                                            University of Padua (Italy)
C                                        {mcaliari,marcov}@math.unipd.it
C
C                                                      Stefano De Marchi
C                                         Department of Computer Science
C                                           University of Verona (Italy)
C                                              stefano.demarchi@univr.it
C
C                                                        June 19th, 2007
C
C       This subroutine computes the coefficients, in the basis of the 
C     normalized Chebyshev polynomials T_j(x)T_{k-j}(y), 0 <= j <= k <= 
C     DEG, T_0(x)=1, T_j(x) = sqrt(2) * cos(j * acos(x)), of the 
C     interpolation polynomial of degree DEG of the function values FPD 
C     at the set of NPD Padua points (PD1,PD2) in the plane. The 
C     interpolant may be evaluated at an arbitrary point by function 
C     PD2VAL. PD1, PD2 and WPD are the Padua points and weights computed
C     by PDPTS.
C
C     ******************************************************************
C
C     PARAMS type                i=input, o=output, a=auxiliary           
C     
C     DEG    integer             i degree of approximation
C     DEGMAX integer             i maximum degree allowed. DEGMAX + 1 is
C                                  the leading dimension of T1, T2 and 
C                                  C0
C     NPD    integer             i number of Padua points
C     PD1    double(NPD)         i array of the first coordinates of 
C                                  Padua points
C     PD2    double(NPD)         i array of the second coordinates of 
C                                  Padua points
C     WPD    double(NPD)         i array of the weights
C     FPD    double(NPD)         i array of the values of the function  
C                                  at Padua points
C     T1     double(0:DEG,NPD)   a Normalized Chebyshev polynomials at
C                                  the first coordinate of Padua points
C     T2     double(0:DEG,NPD)   a Normalized Chebyshev polynomials at
C                                  the second coordinate of Padua points
C     C0     double(0:DEG,0:DEG) o matrix C0 of the coefficients
C     ESTERR double              o estimated error
C
C     ******************************************************************
C
      INTEGER DEG,DEGMAX,NPD
      DOUBLE PRECISION ESTERR
      DOUBLE PRECISION PD1(NPD),PD2(NPD),WPD(NPD),FPD(NPD),
     1     T1(0:DEGMAX,NPD),T2(0:DEGMAX,NPD),C0(0:DEGMAX,0:DEG)
C
C     LOCALS
C
      INTEGER I,J
      DOUBLE PRECISION DDOT
      EXTERNAL DDOT
C
C     Compute Chebyshev polynomials (T1 is scaled) at Padua points
C     
      DO 10 I = 1,NPD
         CALL CHEB(DEG,PD1(I),T1(0,I),WPD(I) * FPD(I))
         CALL CHEB(DEG,PD2(I),T2(0,I),1.0D0)
 10   CONTINUE
C     
C     B = T1 * T2'
C     
      CALL DGEMM('N','T',DEG + 1,DEG + 1,NPD,1.0D0,
     1        T1,DEGMAX + 1,T2,DEGMAX + 1,0.0D0,C0,DEGMAX + 1)
C
C     C0 IS NOT COMPUTED, BUT THE ZERO PART IS NOT USED IN PD2VAL
C
C     
C     Halve the correct term in C0
      C0(DEG,0) = C0(DEG,0) / 2
C
C     Estimated error
C
      ESTERR = 0.0D0
      DO 20 J = 0,2
         DO 30 I = 0,DEG - J
            ESTERR = ESTERR + ABS(C0(I,DEG - I - J))
 30      CONTINUE
 20   CONTINUE
      ESTERR = 2 * ESTERR
      END
